// For a full description of the model economy here, see equations (1) - (6) in the two slides 
// starting with the one titled 'Collecting Equilibrium Conditions'. Those six equations are 
// the private sector equilibrium conditions. The model below also has two equations characterizing
// the law of motion of the two shocks, as well as a nonlinear representation of the Taylor rule.

//The price distortion term, pstar, appears to move substantially more in the second order perturbation than it does in the
// first order perturbation (actually, it can fall to as low as 0.95, which means a 5% drop in output...the linearization
// implies pstar is always exactly unity). 

//Notice that in the second order perturbation (both impulse responses and simulation) the price distortion term, p-star
//does not move much. However, if you replace the technology shock with the second order autoregression, which is far more
//persistent, then p-star moves substantially in the simulation (to do the replacement, comment out line 48 and 
//comment in line 49). Also, note how different the impulse responses are generally when you replace the time series 
//representation of the technology shock. Make sure you understand the intuition behind the differences (hint: recall
//the differences we got in the handout between the representation in terms of a_t and (a_t-a_t-1)).

// Endogenous variables
var R tau pibar a pstar K F N C;

// Innovations
varexo eps_A eps_tau;

// PARAMETERS
parameters nbeta epsil phi rho nu pitarget theta alphaarg rholambda phi_pie phi_x;

nbeta = 0.97;
epsil = 5;
phi=1;
rho = 0.2;
alphaarg = 0.0;
nu=1/epsil; //Do not change this
pitarget=1.; //Do not change this
theta = 0.75;
rholambda = 0.5;
phi_pie = 1.5;
phi_x = 0;

[R_SS,tau_SS,pibar_SS,A_SS,pstar_SS,K_SS,F_SS,N_SS,C_SS] = steadysCALVO(nbeta,epsil,phi,nu,pitarget,theta,alphaarg,rholambda,phi_pie,phi_x);

model;

#Rsteady = 1/nbeta;
1/C - (nbeta/C(+1))*(R/pibar(+1));
1 + (pibar(+1)^(epsil-1))*nbeta*theta*F(+1) -F;
C - pstar*exp(a)*N;
F*(((1-theta*pibar^(epsil-1))/(1-theta))^(1/(1-epsil))) - K;
(1 - nu)*(epsil/(epsil-1))*C*exp(tau)*(N^phi)/exp(a) + nbeta*theta*(pibar(+1)^epsil)*K(+1) - K;
1/pstar - ((1-theta)*(((1-theta*(pibar)^(epsil-1))/(1-theta))^(epsil/(epsil-1))) + theta*(pibar^epsil)/pstar(-1));
R/Rsteady = ((R(-1)/Rsteady)^alphaarg) * exp( (1-alphaarg) * ( phi_pie*(pibar-1) + phi_x*( log(C) - (a - tau/(1+phi)) ) ) );
a = rho*a(-1) + eps_A;
//a = 1.85*a(-1) - 0.855*a(-2) + eps_A;
tau = rholambda*tau(-1)+eps_tau;

end;

initval;
R=R_SS;
tau=tau_SS; 
pibar=pibar_SS; 
a=log(A_SS); 
pstar=pstar_SS; 
K=K_SS; 
F=F_SS;
N=N_SS;
C=C_SS;
end;


steady;
check;


shocks;
var eps_A;
stderr .01;
var eps_tau;
stderr .01;
end;


// first do impulse responses and stochastic simulation by linear approximation
set_dynare_seed(1);

// option drop is used for computing second moments based on the simulations triggered by periods.
// stochastic simulations are also done to compute irf when order > 1 and drop is used here too.
// the first drop observations are dropped from the stochastic simulations when the second moments and/or irf's are computed.
// by default, drop = 100.

stoch_simul(order=1,irf=20,periods=50,nograph,drop=0) R tau pibar a pstar K F N C;

// save the irf's and the stochastic simulations
[Rlinear,taulinear,pibarlinear,Alinear,pstarlinear,Klinear,Flinear,Nlinear,Clinear]=transf(R,tau,pibar,a,pstar,K,F,N,C);
[replinearR_eps_A,replineartau_eps_A,replinearpibar_eps_A,replinearA_eps_A,replinearpstar_eps_A,replinearK_eps_A, ...
    replinearF_eps_A,replinearN_eps_A,replinearC_eps_A,replinearR_eps_tau,replineartau_eps_tau,replinearpibar_eps_tau, ...
    replinearA_eps_tau, replinearpstar_eps_tau, replinearK_eps_tau , replinearF_eps_tau , replinearN_eps_tau, replinearC_eps_tau] ...
    = transf1( R_eps_A,tau_eps_A ,pibar_eps_A ,a_eps_A , pstar_eps_A,K_eps_A, F_eps_A,N_eps_A ,C_eps_A , R_eps_tau, ...
    tau_eps_tau,pibar_eps_tau ,a_eps_tau ,pstar_eps_tau ,  K_eps_tau,F_eps_tau ,N_eps_tau ,C_eps_tau);

// now do everything with order = 2 and pruning
set_dynare_seed(1);
stoch_simul(order=2,pruning,periods=50,replic=1,nograph,drop=0) R tau pibar a pstar K F N C;
[RPrunesim,tauPrunesim,pibarPrunesim,APrunesim,pstarPrunesim,KPrunesim,FPrunesim,NPrunesim,CPrunesim]= ...
     transf(R,tau,pibar,a,pstar,K,F,N,C);

stoch_simul(order=2,irf=20,pruning,replic=100,nograph) R tau pibar a pstar K F N C;
[repPruneR_eps_A,repPrunetau_eps_A,repPrunepibar_eps_A,repPruneA_eps_A,repPrunepstar_eps_A,repPruneK_eps_A, ...
    repPruneF_eps_A,repPruneN_eps_A,repPruneC_eps_A,repPruneR_eps_tau,repPrunetau_eps_tau,repPrunepibar_eps_tau, ...
    repPruneA_eps_tau, repPrunepstar_eps_tau, repPruneK_eps_tau , repPruneF_eps_tau , repPruneN_eps_tau, repPruneC_eps_tau] ...
    = transf1( R_eps_A,tau_eps_A ,pibar_eps_A ,a_eps_A , pstar_eps_A,K_eps_A, F_eps_A,N_eps_A ,C_eps_A , R_eps_tau, ...
    tau_eps_tau,pibar_eps_tau ,a_eps_tau ,pstar_eps_tau ,  K_eps_tau,F_eps_tau ,N_eps_tau ,C_eps_tau);

// now, graph the results
tt=1:length(replinearC_eps_A);
figure
ia=3;
ib=3;
subplot(ia,ib,1)
plot(tt,100*replinearC_eps_A,'o-',tt,100*repPruneC_eps_A,'*-')
legend('linear','order = 2 perturbation')
title('consumption, eps_A shock')
axis tight

subplot(ia,ib,2)
plot(tt,100*replinearpibar_eps_A,'o-',tt,100*repPrunepibar_eps_A,'*-')
title('inflation [pibar], eps_A shock')
axis tight

subplot(ia,ib,3)
plot(tt,100*replinearN_eps_A,tt,100*repPruneN_eps_A,'*-')
title('employment [N], eps_A shock')
axis tight

subplot(ia,ib,4)
plot(tt,100*replinearR_eps_A,tt,100*repPruneR_eps_A,'*-')
title('nominal rate, eps_A shock')
axis tight

subplot(ia,ib,5)
plot(tt,100*replinearA_eps_A,tt,100*repPruneA_eps_A,'*-')
title('technology, eps_A shock')
axis tight

subplot(ia,ib,6)
plot(tt,100*replineartau_eps_A,tt,100*repPrunetau_eps_A,'*-')
title('tau, eps_A shock')
axis tight

subplot(ia,ib,7)
plot(tt,100*replinearpstar_eps_A,tt,100*repPrunepstar_eps_A,'*-')
title('pstar, eps_A shock')
axis tight

subplot(ia,ib,8)
plot(tt,100*replinearK_eps_A,tt,100*repPruneK_eps_A,'*-')
title('K, eps_A shock')
axis tight

subplot(ia,ib,9)
plot(tt,100*replinearF_eps_A,tt,100*repPruneF_eps_A,'*-')
title('F, eps_A shock')
axis tight



figure
ia=3;
ib=3;
subplot(ia,ib,1)
plot(tt,100*replinearC_eps_tau,tt,100*repPruneC_eps_tau,'*-')
legend('linear','order=2 perturbation')
title('consumption, tau shock')
axis tight

subplot(ia,ib,2)
plot(tt,100*replinearpibar_eps_tau,tt,100*repPrunepibar_eps_tau,'*-')
title('inflation [pibar], tau shock')
axis tight

subplot(ia,ib,3)
plot(tt,100*replinearN_eps_tau,tt,100*repPruneN_eps_tau,'*-')
title('employment [N], tau shock')
axis tight

subplot(ia,ib,4)
plot(tt,100*replinearR_eps_tau,tt,100*repPruneR_eps_tau,'*-')
title('nominal rate, tau shock')
axis tight

subplot(ia,ib,5)
plot(tt,100*replinearA_eps_tau,tt,100*repPruneA_eps_tau,'*-')
title('technology, tau shock')
axis tight

subplot(ia,ib,6)
plot(tt,100*replineartau_eps_tau,tt,100*repPrunetau_eps_tau,'*-')
title('tau, tau shock')
axis tight

subplot(ia,ib,7)
plot(tt,100*replinearpstar_eps_tau,tt,100*repPrunepstar_eps_tau,'*-')
title('pstar, tau shock')
axis tight

subplot(ia,ib,8)
plot(tt,100*replinearK_eps_tau,tt,100*repPruneK_eps_tau,'*-')
title('K, tau shock')
axis tight

subplot(ia,ib,9)
plot(tt,100*replinearF_eps_tau,tt,100*repPruneF_eps_tau,'*-')
title('F, tau shock')
axis tight


figure

tt=1:length(C);
ia=3;
ib=3;
subplot(ia,ib,1)
plot(tt,Clinear,tt,CPrunesim,'*-')
legend('linear','order=2 perturbation')
title('consumption')
axis tight

subplot(ia,ib,2)
plot(tt,pibarlinear,tt,pibarPrunesim,'*-')
title('inflation [pibar]')
axis tight

subplot(ia,ib,3)
plot(tt,Nlinear,tt,NPrunesim,'*-')
title('employment [N]')
axis tight

subplot(ia,ib,4)
plot(tt,Rlinear,tt,RPrunesim,'*-')
title('nominal rate')
axis tight

subplot(ia,ib,5)
plot(tt,Alinear,tt,APrunesim,'*-')
title('technology [A]')
axis tight

subplot(ia,ib,6)
plot(tt,taulinear,tt,tauPrunesim,'*-')
title('tau')
axis tight

subplot(ia,ib,7)
plot(tt,pstarlinear,tt,pstarPrunesim,'*-')
title('pstar')
axis tight

subplot(ia,ib,8)
plot(tt,Klinear,tt,KPrunesim,'*-')
title('K')
axis tight

subplot(ia,ib,9)
plot(tt,Flinear,tt,FPrunesim,'*-')
title('F')
axis tight




